;
;    This routine reads Level 2 Swath files, both standard and
;     support files.     However it does not read all the parameters
;     in the data set.     If you need to read more than what this
;     does, please add keyword=parameter, so that old IDL routines can
;     still use this routine.
;
;    If you want to add more optional parameters to read from the swath files,
;     get the list of variables (and their dimensions) using the tool
;     run_dump_hdfeos in ~/AIRS/SUN/bin/.   Please note that a keyword cannot
;     be a substring of another keyword.   For example, we could not use TSurf
;     as a keyword, since TSurfErr is a key word.
;
;    Please note that the standard and the support files have some parameters
;      in different unit, as well as different pressure grids.
;
;    Sung-Yung Lee
;    April 09, 2001
;
pro read_Cloudsat_geoprof, pattern, Profile_time, UTC_start, TAI_start, Latitude, Longitude, Height, $
                       Spacecraft_altitude, DEM_elevation, RayHeader_RangeBinSize, Data_quality, Data_status,  $
                       Data_targetID, SurfaceHeightBin, CPR_Cloud_mask, Gaseous_Attenuation, Radar_Reflectivity, $
                       MODIS_Cloud_Fraction, MODIS_scene_char, MODIS_scene_var, CPR_Echo_Top, sem_NoiseFloor 

   numfp = 0
   filename = findfile ( pattern, count = cnt )
   if cnt ne 1 then begin
      print, 'findfile did not return exactly one file ', cnt, pattern
      if cnt eq 0 then return
   endif

   print, 'read_Cloudsat_geoprof: ', filename[0]

   FAIL = -1
   SWATH_L2_STD = '2B-GEOPROF'

   fid=EOS_SW_OPEN(filename[0], /READ)
   if fid eq FAIL then return

   SWid=EOS_SW_ATTACH(fid, SWATH_L2_STD)
   if SWid eq FAIL then begin
      standard = 1
      print, "Reading "+SWATH_L2_STD
   endif

   ret = EOS_SW_READFIELD(SWid,"Profile_time",Profile_time,EDGE=edge)
   ret = EOS_SW_READFIELD(SWid,"UTC_start",UTC_start,EDGE=edge)
   ret = EOS_SW_READFIELD(SWid,"TAI_start",TAI_start,EDGE=edge)

   ret = EOS_SW_READFIELD(SWid,"Latitude",Latitude)
   ret = EOS_SW_READFIELD(SWid,"Longitude",Longitude)

   ret = EOS_SW_READFIELD(SWid,"Height",Height)
   ret = EOS_SW_READFIELD(SWid,"Spacecraft_altitude",Spacecraft_altitude)
   ret = EOS_SW_READFIELD(SWid,"DEM_elevation",DEM_elevation)
   ret = EOS_SW_READFIELD(SWid,"RayHeader_RangeBinSize",RayHeader_RangeBinSize)
   ret = EOS_SW_READFIELD(SWid,"Data_quality",Data_quality)
   ret = EOS_SW_READFIELD(SWid,"Data_status",Data_status)
   ret = EOS_SW_READFIELD(SWid,"Data_targetID",Data_targetID)
   ret = EOS_SW_READFIELD(SWid,"SurfaceHeightBin",SurfaceHeightBin)
   ret = EOS_SW_READFIELD(SWid,"CPR_Cloud_mask",CPR_Cloud_mask)
   ret = EOS_SW_READFIELD(SWid,"Gaseous_Attenuation",Gaseous_Attenuation)
   ret = EOS_SW_READFIELD(SWid,"Radar_Reflectivity",Radar_Reflectivity)
   ret = EOS_SW_READFIELD(SWid,"MODIS_Cloud_Fraction",MODIS_Cloud_Fraction)
   ret = EOS_SW_READFIELD(SWid,"MODIS_scene_char",MODIS_scene_char)
   ret = EOS_SW_READFIELD(SWid,"MODIS_scene_var",MODIS_scene_var)
   ret = EOS_SW_READFIELD(SWid,"CPR_Echo_Top",CPR_Echo_Top)
   ret = EOS_SW_READFIELD(SWid,"sem_NoiseFloor",sem_NoiseFloor)

   print, "done reading - detaching and closing the file"
   ret = EOS_SW_DETACH(SWid)
   ret = EOS_SW_CLOSE(fid)

end
