;
;    This routine reads Level 2 Swath files, both standard and
;     support files.     However it does not read all the parameters
;     in the data set.     If you need to read more than what this
;     does, please add keyword=parameter, so that old IDL routines can
;     still use this routine.
;
;    If you want to add more optional parameters to read from the swath files,
;     get the list of variables (and their dimensions) using the tool
;     run_dump_hdfeos in ~/AIRS/SUN/bin/.   Please note that a keyword cannot
;     be a substring of another keyword.   For example, we could not use TSurf
;     as a keyword, since TSurfErr is a key word.
;
;    Please note that the standard and the support files have some parameters
;      in different unit, as well as different pressure grids.
;
;    Sung-Yung Lee
;    April 09, 2001
;
pro read_swath_l2, pattern, numfp, numline, $
		tai, lat, lon, pres, tair, h2o, ozo, $
		scan_node_type=scan_node_type, $
		TAirErr=TAirErr, $
		H2OMMRErr=H2OMMRErr, $
		totH2OErr=totH2OErr, $
		O3VMRErr=O3VMRErr, $
		totO3Std=totO3Std, $
		totH2OStd=totH2OStd, $
		totO3Err=totO3Err, $
		MW_ret_used=MW_ret_used, $
		NSurface=NSurf, $
		TSurface=TSurf, $
		TSurf1Ret=TSurf1Ret, $
		tsurf_forecast=ts_forecast, $
		clear_flag_4um=clear_flag_4um, $
		clear_flag_11um=clear_flag_11um, $
		tsurf_diff_4um=tsurf_diff_4um, $
		tsurf_diff_11um=tsurf_diff_11um, $
		spatial_coh_4um=spatial_coh_4um, $
		spatial_coh_11um=spatial_coh_11um, $
		TSurfErr=TSurfErr, $
		nhEmis=nhEmis, $
		freqEmis=freqEmis, $
		emisIR=emisIR, $
		emisInit=emisInit, $
		rhoIR=rhoIR, $
		CldClearParam=CldClearParam, $
		zenang=zenang, $
		sfcTbMW=sfcTbMW, $
		emismw=emismw, $
		TAirMWOnly=TAirMWOnly, $
		CldFrc=CldFrc, $
                CloudFrcErr=CloudFrcErr, $
		PCldTop=PCldTop, $
                PCloudTopErr=PCloudTopErr, $
		TCldTop=TCldTop, $
                TCloudTopErr=TCloudTopErr, $
		totCldH2O=totCldH2O, $
		lwCD=lwCD, $
		TAir1Ret=TAir1Ret, $
		H2OCD1Ret=H2OCD1Ret, $
		O3CDInit=O3CDInit, $
		cIW=cIW, $
		PSurf=PSurf, $
		GP_Surface=GP_Surface, $
		GP_Height=GP_Height, $
		LandFrac=LandFrac, $
		topog=topog, $
		CO2ppmv=CO2ppmv, $
		sun_glint_distance=sun_glint_distance, $
		invalid=invalid, $
		mwresidual_amsua=mwresidual_amsua, $
		mwresidual_hsb=mwresidual_hsb, $
		MWSurfClass=MWSurfClass, $
		rain_rate=rain_rate, $
		Precip_Corr=Precip_Corr, $
		fov_clear_flag=fov_clear_flag, $
		clear_flag_4window=clear_flag_4window, $
		tsurf_diff_4window=tsurf_diff_4window, $
		prelim_clear_flag=prelim_clear_flag, $
		cc_score=cc_score, $
		olr=olr, $
		clrolr=clrolr, $
		solzen=solzen, $
		final_clear_flag=clear_flag, $
		retqa=retqa, $
		retrieval_type=retrieval_type, $
		TSAir=TSAir,  $
                CloudOLRFlag=CloudOLRFlag, $
                start_Time=start_Time

   numfp = 0
   filename = findfile ( pattern, count = cnt )
   if cnt ne 1 then begin
      print, 'findfile did not return exactly one file ', cnt, pattern
      if cnt eq 0 then return
   endif

   print, 'read_swath_l2: ', filename[0]

   FAIL = -1
   SWATH_L2_STD = 'L2_Standard_atmospheric&surface_product'
   SWATH_L2_SUP = 'L2_Support_atmospheric&surface_product'

   fid=EOS_SW_OPEN(filename[0], /READ)
   if fid eq FAIL then return

   SWid=EOS_SW_ATTACH(fid, SWATH_L2_STD)
   if SWid eq FAIL then begin
      SWid=EOS_SW_ATTACH(fid, SWATH_L2_SUP)
      if SWid eq FAIL then return
      standard = 0
      print, "Reading "+SWATH_L2_SUP
   endif else begin
      standard = 1
      print, "Reading "+SWATH_L2_STD
   endelse

   ret = EOS_SW_READFIELD(SWid,"Time",tai,EDGE=edge)
   sz=size(tai)
   numfp=sz[1]
   numline=sz[2]

   ret = EOS_SW_READFIELD(SWid,"Latitude",lat)
   ret = EOS_SW_READFIELD(SWid,"Longitude",lon)

   if arg_present(scan_node_type) then begin
      ret = EOS_SW_READFIELD(SWid,"scan_node_type",scan_node_type)
   endif

   if arg_present(start_Time) then begin
      ret = EOS_SW_READFIELD(SWid,"start_Time",start_Time)
   endif

   if standard eq 1 then begin
      ret = EOS_SW_READFIELD(SWid,"pressStd", pres)
      ret = EOS_SW_READFIELD(SWid,"TAirStd", tair)
      ret = EOS_SW_READFIELD(SWid,"H2OMMRStd", h2o)
      ret = EOS_SW_READFIELD(SWid,"O3VMRStd", ozo)

      if arg_present(TSAir) then begin
         print, "reading TSurfAir"
         ret = EOS_SW_READFIELD(SWid,"TSurfAir", TSAir)
      endif

      if arg_present(TAirErr) then begin
         print, "reading the TAir Error Estimate"
         ret = EOS_SW_READFIELD(SWid,"TAirStdErr", TAirErr)
      endif

      if arg_present(totH2OErr) then begin
         print, "reading the Total Water Vapor Error Estmate"
         ret = EOS_SW_READFIELD(SWid,"totH2OStdErr", totH2OErr)
      endif

      if arg_present(H2OMMRErr) then begin
         print, "reading the H2OMMR Error Estmate"
         ret = EOS_SW_READFIELD(SWid,"H2OMMRStdErr", H2OMMRErr)
      endif

      if arg_present(totO3Std) then begin
         print, "reading the Total Ozone Burden"
         ret = EOS_SW_READFIELD(SWid,"totO3Std", totO3Std)
      endif

      if arg_present(totO3Err) then begin
         print, "reading the Total Ozone Burden Error Estmate"
         ret = EOS_SW_READFIELD(SWid,"totO3StdErr", totO3Err)
      endif

      if arg_present(O3VMRErr) then begin
         print, "reading the O3VMR Err Estimate"
         ret = EOS_SW_READFIELD(SWid,"O3VMRStdErr", O3VMRErr)
      endif

      if arg_present(sfcTbMW) then begin
         print, "reading the MW surface brightness"
         ret = EOS_SW_READFIELD(SWid,"sfcTbMWStd", sfcTbMW)
      endif

      if arg_present(emismw) then begin
         print, "reading the MW emissivities"
         ret = EOS_SW_READFIELD(SWid,"EmisMWStd", emismw)
      endif

      if arg_present(emisIR) then begin
         print, "reading the IR emissivity"
         ret = EOS_SW_READFIELD(SWid,"emisIRStd", emisIR)
      endif

      if arg_present(emisInit) then begin
         print, "reading the initial IR emissivity"
         ret = EOS_SW_READFIELD(SWid,"emisIRInit", emisInit)
      endif

      if arg_present(rhoIR) then begin
         print, "reading the IR reflectivity"
         ret = EOS_SW_READFIELD(SWid,"rhoIRStd", rhoIR)
      endif

      if arg_present(CldFrc) then begin
         print, "reading the cloud fraction"
         ret = EOS_SW_READFIELD(SWid,"CldFrcStd", CldFrc)
      endif

      if arg_present(CloudFrcErr) then begin
         print, "reading the cloud fraction error"
         ret = EOS_SW_READFIELD(SWid,"CldFrcStdErr", CloudFrcErr)
      endif

      if arg_present(TCldTop) then begin
         print, "reading the cloud top temperature"
         ret = EOS_SW_READFIELD(SWid,"TCldTopStd", TCldTop)
      endif

      if arg_present(TCloudTopErr) then begin
         print, "reading the cloud top temperature error"
         ret = EOS_SW_READFIELD(SWid,"TCldTopStdErr", TCloudTopErr)
      endif

      if arg_present(PCldTop) then begin
         print, "reading the cloud top pressure"
         ret = EOS_SW_READFIELD(SWid,"PCldTopStd", PCldTop)
      endif

      if arg_present(PCloudTopErr) then begin
         print, "reading the cloud top pressure error"
         ret = EOS_SW_READFIELD(SWid,"PCldTopStdErr", PCloudTopErr)
      endif

      if arg_present(nhEmis) then begin
         print, "reading the number of emis hinge points"
         ret = EOS_SW_READFIELD(SWid,"numHingeSurf", nhEmis)
      endif

      if arg_present(freqEmis) then begin
         print, "reading the frequencies for emissivity"
         ret = EOS_SW_READFIELD(SWid,"freqEmis", freqEmis)
      endif

      if arg_present(totCldH2O) then begin
         print, "reading the total cloud liquid water"
         ret = EOS_SW_READFIELD(SWid,"totCldH2OStd", totCldH2O)
      endif

      if arg_present(totH2OStd) then begin
         print, "reading the total precipitable water"
         ret = EOS_SW_READFIELD(SWid,"totH2OStd", totH2OStd)
      endif

      if arg_present(GP_Surface) then begin
         print, "reading the geopotential at the surface"
         ret = EOS_SW_READFIELD(SWid,"GP_Surface", GP_Surface)
      endif

      if arg_present(GP_Height) then begin
         print, "reading the geopotential height"
         ret = EOS_SW_READFIELD(SWid,"GP_Height", GP_Height)
      endif

      if arg_present(CldClearParam) then begin
         print, "reading the cloud clearing parameter (eta)"
         ret = EOS_SW_READFIELD(SWid,"CldClearParamStd", CldClearParam)
      endif

      if arg_present(MW_ret_used) then begin
         print, "reading the flag MW_ret_used"
         ret = EOS_SW_READFIELD(SWid,"MW_ret_used", MW_ret_used)
      endif

      if arg_present(NSurf) then begin
         print, "reading the surface level pointer"
         ret = EOS_SW_READFIELD(SWid,"nSurfStd", NSurf)
      endif

      if arg_present(clear_flag_4um) then begin
         print, "reading the 4 micron window channel based clear flag"
         ret = EOS_SW_READFIELD(SWid,"clear_flag_4um", clear_flag_4um)
      endif

      if arg_present(clear_flag_11um) then begin
         print, "reading the 11 micron window channel based clear flag"
         ret = EOS_SW_READFIELD(SWid,"clear_flag_11um", clear_flag_11um)
      endif

      if arg_present(clear_flag_4window) then begin
         print, "reading the 4 micron window difference"
         ret = EOS_SW_READFIELD(SWid,"clear_flag_4window", clear_flag_4window)
      endif
   endif else begin
      ret = EOS_SW_READFIELD(SWid,"pressSupp", pres)
      ret = EOS_SW_READFIELD(SWid,"TAirSup", tair)
      ret = EOS_SW_READFIELD(SWid,"H2OCDSup", h2o)
      ret = EOS_SW_READFIELD(SWid,"O3CDSup", ozo)
      if arg_present(TAirMWOnly) then begin
         print, "reading MW only TAir"
         ret = EOS_SW_READFIELD(SWid,"TAirMWOnly", TAirMWOnly)
      endif

      if arg_present(TAir1Ret) then begin
         print, "reading the Initial Guess Tair"
         ret = EOS_SW_READFIELD(SWid,"TAir1Ret", TAir1Ret)
      endif

      if arg_present(H2OCD1Ret) then begin
         print, "reading the Initial Guess H2O CD"
         ret = EOS_SW_READFIELD(SWid,"H2OCD1Ret", H2OCD1Ret)
      endif

      if arg_present(O3CDInit) then begin
         print, "reading the Initial Guess Ozone CD"
         ret = EOS_SW_READFIELD(SWid,"O3CDInit", O3CDInit)
      endif

      if arg_present(TSurf1Ret) then begin
         print, "reading First Tsurf"
         ret = EOS_SW_READFIELD(SWid,"TSurf1Ret", TSurf1Ret)
      endif

      if arg_present(mwresidual_amsua) then begin
         print, "reading the MWresidual_AMSUA"
         ret = EOS_SW_READFIELD(SWid,"MWresidual_AMSUA", mwresidual_amsua)
      endif

      if arg_present(lwCD) then begin
         print, "reading the Cloud liquid water profile"
         ret = EOS_SW_READFIELD(SWid,"lwCDSup", lwCD)
      endif

      if arg_present(cIW) then begin
         print, "reading the Ice/Water flag for Cloud liquid water profile"
         ret = EOS_SW_READFIELD(SWid,"cIWSup", cIW)
      endif

      if arg_present(MWSurfClass) then begin
         print, "reading the MW surface class"
         ret = EOS_SW_READFIELD(SWid,"MWSurfClass", MWSurfClass)
      endif

      if arg_present(rain_rate) then begin
         print, "reading the rain rate"
         ret = EOS_SW_READFIELD(SWid,"rain_rate", rain_rate)
      endif

      if arg_present(mwresidual_hsb) then begin
         print, "reading the MWresidual_HSB"
         ret = EOS_SW_READFIELD(SWid,"MWresidual_HSB", mwresidual_hsb)
      endif

      if arg_present(Precip_Corr) then begin
         Precip_Corr = fltarr(numfp,numline,6)
         print, "reading the Precipitation Correction Term for AMSU channels 4 - 9"
         ret = EOS_SW_READFIELD(SWid,"AMSU_A_4_Precip_Corr", buff)
         Precip_Corr(*,*,0) = buff
         ret = EOS_SW_READFIELD(SWid,"AMSU_A_5_Precip_Corr", buff)
         Precip_Corr(*,*,1) = buff
         ret = EOS_SW_READFIELD(SWid,"AMSU_A_6_Precip_Corr", buff)
         Precip_Corr(*,*,2) = buff
         ret = EOS_SW_READFIELD(SWid,"AMSU_A_7_Precip_Corr", buff)
         Precip_Corr(*,*,3) = buff
         ret = EOS_SW_READFIELD(SWid,"AMSU_A_8_Precip_Corr", buff)
         Precip_Corr(*,*,4) = buff
         ret = EOS_SW_READFIELD(SWid,"AMSU_A_9_Precip_Corr", buff)
         Precip_Corr(*,*,5) = buff
      endif

      if arg_present(NSurf) then begin
         print, "reading the surface level pointer"
         ret = EOS_SW_READFIELD(SWid,"nSurfSup", NSurf)
      endif

      if arg_present(clrolr) then begin
         print, "reading the Clear Sky Outoging longwave radiation"
         ret = EOS_SW_READFIELD(SWid,"clrolr", clrolr)
      endif

      if arg_present(olr) then begin
         print, "reading the Outoging longwave radiation"
         ret = EOS_SW_READFIELD(SWid,"olr", olr)
      endif

      if arg_present(cc_score) then begin
         print, "reading the initial CC score"
         ret = EOS_SW_READFIELD(SWid,"Initial_CC_score", cc_score)
      endif

      if arg_present(fov_clear_flag) then begin
         print, "reading the fov_clear_flag "
         ret = EOS_SW_READFIELD(SWid,"fov_clear_flag", fov_clear_flag)
      endif

      if arg_present(prelim_clear_flag) then begin
         print, "reading the prelim_clear_flag "
         ret = EOS_SW_READFIELD(SWid,"prelim_clear_flag", prelim_clear_flag)
      endif

      if arg_present(ts_forecast) then begin
         print, "reading forecast surface skin temperature"
         ret = EOS_SW_READFIELD(SWid,"tsurf_forecast", ts_forecast)
      endif

      if arg_present(tsurf_diff_4um) then begin
         print, "reading the 4 micron window channel difference to SST"
         ret = EOS_SW_READFIELD(SWid,"tsurf_diff_4um", tsurf_diff_4um)
      endif

      if arg_present(tsurf_diff_11um) then begin
         print, "reading the 11 micron window channel difference to SST"
         ret = EOS_SW_READFIELD(SWid,"tsurf_diff_11um", tsurf_diff_11um)
      endif

      if arg_present(spatial_coh_4um) then begin
         print, "reading the 4 micron window channel coherence"
         ret = EOS_SW_READFIELD(SWid,"spatial_coh_4um", spatial_coh_4um)
      endif

      if arg_present(spatial_coh_11um) then begin
         print, "reading the 11 micron window channel coherence"
         ret = EOS_SW_READFIELD(SWid,"spatial_coh_11um", spatial_coh_11um)
      endif

      if arg_present(CO2ppmv) then begin
         print, "reading the CO2 mixing ratio"
         ret = EOS_SW_READFIELD(SWid,"CO2ppmv", CO2ppmv)
      endif

   endelse

   if arg_present(PSurf) then begin
      print, "reading Surface Pressure"
      ret = EOS_SW_READFIELD(SWid,"PSurfStd", Psurf)
   endif

   if arg_present(TSurf) then begin
      print, "reading surface skin temperature"
      ret = EOS_SW_READFIELD(SWid,"TSurfStd", TSurf)
   endif

   if arg_present(TSurfErr) then begin
      print, "reading surface skin temperature error estimate "
      ret = EOS_SW_READFIELD(SWid,"TSurfStdErr", TSurfErr)
   endif

   if arg_present(LandFrac) then begin
      print, "reading the land fraction "
      ret = EOS_SW_READFIELD(SWid,"landFrac", LandFrac)
   endif

   if arg_present(solzen) then begin
      print, "reading the solar zenith angle "
      ret = EOS_SW_READFIELD(SWid,"solzen", solzen)
   endif

   if arg_present(topog) then begin
      print, "reading the topography"
      ret = EOS_SW_READFIELD(SWid,"topog", topog)
   endif

   if arg_present(sun_glint_distance) then begin
      print, "reading the sun glint distance"
      ret = EOS_SW_READFIELD(SWid,"sun_glint_distance", sun_glint_distance)
   endif

   if arg_present(zenang) then begin
      print, "reading satellite zenith angle"
      ret = EOS_SW_READFIELD(SWid,"satzen", zenang)
   endif

   if arg_present(final_clear_flag) then begin
      print, "reading the clear flag"
      ret = EOS_SW_READFIELD(SWid,"clear_flag", clear_flag)
   endif

   if arg_present(retrieval_type) then begin
      print, "reading the flag retrieval_type"
      ret = EOS_SW_READFIELD(SWid,"retrieval_type", retrieval_type)
   endif

   if arg_present(RetQA) then begin
      print, "reading the retrieval QA flag"
      ret = EOS_SW_READFIELD(SWid,"RetQAFlag", RetQA)
   endif

   if arg_present(invalid) then begin
      print, "reading the invalid flag"
      ret = EOS_SW_READFIELD(SWid,"invalid", invalid)
   endif

   if arg_present(CloudOLRFlag) then begin
      print, "reading the retrieval Qual Cloud OLR flag"
      ret = EOS_SW_READFIELD(SWid,"Qual_Cloud_OLR", CloudOLRFlag)
   endif

   print, "done reading - detaching and closing the file"
   ret = EOS_SW_DETACH(SWid)
   ret = EOS_SW_CLOSE(fid)

end
